using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace Meilensteine
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        protected override void OnPaint(PaintEventArgs e)
        {
            base.OnPaint(e);
            e.Graphics.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.HighQuality;

            const int steps = 1000;
            PointF[] points = new PointF[steps];
            for (int i = 0; i < steps; i++)
            {
                points[i] = curve(i / (double)(steps-1));
            }
            e.Graphics.DrawLines(Pens.Black, points);

            double[] arcs = new double[steps];
            double sum = 0.0;
            for (int i = 1; i < steps; i++)
            {
                double dx = points[i].X-points[i-1].X;
                double dy = points[i].Y - points[i - 1].Y;
                sum += Math.Sqrt(dx*dx+dy*dy);
                arcs[i] = sum;
            }

            const double grid = 80.0;
            double milestone = grid;
            for (int i = 1; i < steps; i++)
            {
                while (arcs[i] >= milestone)
                {
                    double dx = points[i].X - points[i - 1].X;
                    double dy = points[i].Y - points[i - 1].Y;
                    double len = Math.Sqrt(dx * dx + dy * dy);
                    double diff = milestone - arcs[i-1];
                    double r = diff / len;

                    float x = points[i - 1].X + (float)(dx * r);
                    float y = points[i - 1].Y + (float)(dy * r);

                    //e.Graphics.FillEllipse(Brushes.Red, x-2, y-2, 4, 4);

                    const double gridLen = 5.0;
                    PointF p1 = new PointF(x + (float)(gridLen * dy / len), y - (float)(gridLen*dx / len));
                    e.Graphics.DrawLine(Pens.Black, new PointF(x, y), p1);

                    SizeF s = e.Graphics.MeasureString(milestone.ToString(), new Font("Arial", 9.0f));
                    if (dy >= 0.0)
                    {
                        e.Graphics.DrawString(milestone.ToString(), new Font("Arial", 9.0f), Brushes.Blue, p1.X, p1.Y - s.Height * 0.5f);
                    }
                    else
                    {
                        e.Graphics.DrawString(milestone.ToString(), new Font("Arial", 9.0f), Brushes.Blue, p1.X - s.Width, p1.Y - s.Height * 0.5f);
                    }
                    milestone += grid;
                }
            }
        }

        private PointF curve(double t)
        {
            PointF p = new PointF();
            p.X = 200.0f + 150.0f * (float)(t * Math.Cos(20 * Math.PI * t));
            p.Y = 200.0f + 150.0f * (float)(t * Math.Sin(20 * Math.PI * t));
            return p;
        }
    }
}